//+------------------------------------------------------------------+
//|                                               Special EURGBP.mq4 |
//|                                       Copyright  2008, Mas Awoo |
//+------------------------------------------------------------------+
#include <stdlib.mqh>
#property copyright "Copyright  2008, Mas Awoo"
#property link      "mailto:mas_awoo@yahoo.com"

extern int MAGICMA                      = 31457879;
extern double Lots                      = 0.1;
extern double StopLoss                  = 1111;
extern int    Slippage                  = 3; 
extern double TakeProfit                = 1111;
extern string ____Money_Management____  = "___________________________________________________";
extern double Risk                      = 0.03;
extern double MaximumRisk               = 0.5;
extern double IncreaseFactor            = 3;
extern string _____Stochastic______     = "___________________________________________________";
extern double Stoch_Entry_Buy           = 20;
extern double Stoch_Entry_Sell          = 80;
extern int    K_Period                  = 14;
extern int    D_Period                  = 7;
extern int    Slowing_Period            = 1;
//+------------------------------------------------------------------+
//| Calculate open positions                                         |
//+------------------------------------------------------------------+
int CalculateCurrentOrders(string symbol)
  {
   int buys=0,sells=0;
//----
   for(int i=0;i<OrdersTotal();i++)
     {
      if(OrderSelect(i,SELECT_BY_POS,MODE_TRADES)==false) break;
      if(OrderSymbol()==Symbol() && OrderMagicNumber()==MAGICMA)
        {
         if(OrderType()==OP_BUY)  buys++;
         if(OrderType()==OP_SELL) sells++;
        }
     }
//---- return orders volume
   if(buys>0) return(buys);
   if(sells>0) return(sells);
  }
//+------------------------------------------------------------------+
//| Calculate optimal lot size                                       |
//+------------------------------------------------------------------+
double LotsOptimized()
  {
   double lot=Lots,MaxLot=MarketInfo(Symbol(),MODE_MAXLOT), a;
   int    orders=HistoryTotal();     // history orders total
   int    losses=0, profit=0;                  // number of losses orders without a break
//---- select lot size
 if (Lots<0.1)a=2;
 else a=1;
 lot=NormalizeDouble(AccountFreeMargin()*Risk/1000.0,a);
 double MaxLotSize=NormalizeDouble(AccountFreeMargin()*MaximumRisk/1000.0,a);
//---- calcuulate number of losses orders without a break
   if(IncreaseFactor>0)
     {
      for(int i=orders-1;i>=0;i--)
        {
         if(OrderSelect(i,SELECT_BY_POS,MODE_HISTORY)==false) { Print("Error in history!"); break; }
         if(OrderSymbol()!=Symbol() || OrderType()>OP_SELL) continue;
         //----
         if(OrderProfit()>0) break;
         if(OrderProfit()<0) losses++;
        }
      if(losses>1) lot=NormalizeDouble(AccountFreeMargin()*Risk/1000.0,a)*IncreaseFactor;
     }
//---- return lot size
   if(lot<Lots) lot=Lots;
   if(lot>MaxLot && MaxLotSize<MaxLot ) lot=MaxLotSize;
   if(lot>MaxLot && MaxLotSize>MaxLot ) lot=MaxLot;
   return(lot);
  }
  
//+------------------------------------------------------------------+
//| Check for open order conditions                                  |
//+------------------------------------------------------------------+
void CheckForOpen()
 {   
 int res; 
 double St,Sts;
 //---------------------------------------------
   if (Volume[0]>1)return;
 //-----------------------------------------------

//---- get Moving Average
   St=iStochastic(NULL,0,K_Period,D_Period,Slowing_Period,MODE_SMA,0,MODE_MAIN,0);
   Sts=iStochastic(NULL,0,K_Period,D_Period,Slowing_Period,MODE_SMA,0,MODE_SIGNAL,0);
//---- buy conditions  
    if (St<Stoch_Entry_Buy)
     {
      res=OrderSend(Symbol(),OP_BUY,LotsOptimized(),Ask,Slippage,Ask-StopLoss*Point,Ask+TakeProfit*Point,"Mas Awoo",MAGICMA,0,Blue);
      return;
     }
//---- sell conditions

    if (St>Stoch_Entry_Sell)
     {     
      res=OrderSend(Symbol(),OP_SELL,LotsOptimized(),Bid,Slippage,Bid+StopLoss*Point,Bid-TakeProfit*Point,"Mas Awoo",MAGICMA,0,Red);
      return;
     }
//---- 
 }

//+------------------------------------------------------------------+
//| Check for close order conditions                                 |
//+------------------------------------------------------------------+
void CheckForClose()
  {
 double St,Sts;
//---- go trading only for first tiks of new bar
 if (Volume[0]>10)return;
 
//---- get Moving Average
   St=iStochastic(NULL,0,K_Period,D_Period,Slowing_Period,MODE_SMA,0,MODE_MAIN,0);
   Sts=iStochastic(NULL,0,K_Period,D_Period,Slowing_Period,MODE_SMA,0,MODE_SIGNAL,0);
//----
   for(int i=0;i<OrdersTotal();i++)
     {
      OrderSelect(i,SELECT_BY_POS,MODE_TRADES);
      if(OrderMagicNumber()==MAGICMA && OrderSymbol()==Symbol())
      //---- check order type 
      if(OrderType()==OP_BUY)
        {
    if (Sts>Stoch_Entry_Sell)
          OrderClose(OrderTicket(),OrderLots(),Bid,3,White);
         break;
        }
      if(OrderType()==OP_SELL)
        {       
    if (Sts<Stoch_Entry_Buy)
          OrderClose(OrderTicket(),OrderLots(),Ask,3,White);
         break;
        }
     }
//----
  }

//+------------------------------------------------------------------+
//| Start function                                                   |
//+------------------------------------------------------------------+
void start()
  {
//---- check for history and trading
   if(Bars<100 || IsTradeAllowed()==false) return;
//---- calculate open orders by current symbol
    if(CalculateCurrentOrders(Symbol())==0) CheckForOpen();
    else CheckForClose();
//----
  }
//+------------------------------------------------------------------+